@@ -99,11 +99,15 @@ class GrizzlyContextScenarioValidation:
     avg_response_time: Optional[int] = field(init=False, default=None)
     response_time_percentile: Optional[GrizzlyContextScenarioResponseTimePercentile] = field(init=False, default=None)
 
+@dataclass(unsafe_hash=True)
+class GrizzlyContextScenarioUser:
+    class_name: str = field(init=False, hash=True)
+    weight: int = field(init=False, hash=True, default=1)
 
 @dataclass(unsafe_hash=True)
 class GrizzlyContextScenario:
     name: str = field(init=False, hash=True)
-    user_class_name: str = field(init=False, hash=True)
+    user: GrizzlyContextScenarioUser = field(init=False, hash=False, compare=False, default_factory=GrizzlyContextScenarioUser)
     _identifier: Optional[str] = field(init=False, hash=True, default=None)
     iterations: int = field(init=False, repr=False, hash=False, compare=False, default=1)
 
@@ -159,7 +163,7 @@ class GrizzlyContextSetup:
     global_context: Dict[str, Any] = field(init=False, repr=False, hash=False, compare=False, default_factory=dict)
 
     user_count: int = field(init=False, default=0)
-    spawn_rate: Optional[int] = field(init=False, default=None)
+    spawn_rate: Optional[float] = field(init=False, default=None)
     timespan: Optional[str] = field(init=False, default=None)
 
     statistics_url: Optional[str] = field(init=False, default=None)
