@@ -1,4 +1,4 @@
-from enum import Enum, auto
+from enum import Enum
 from typing import Callable, Optional, Tuple, Any, Union, Dict, TypeVar, List, Type, Generic, Set, cast
 from importlib import import_module
 
@@ -7,6 +7,7 @@ from locust.clients import ResponseContextManager
 from locust.user.users import User
 from gevent.lock import Semaphore
 
+from grizzly_extras.transformer import TransformerContentType
 
 class ResponseTarget(Enum):
     METADATA = 0
@@ -18,13 +19,6 @@ class ResponseAction(Enum):
     SAVE = 1
 
 
-class ResponseContentType(Enum):
-    GUESS = 0
-    JSON = auto()
-    XML = auto()
-    PLAIN = auto()
-
-
 class RequestDirection(Enum):
     FROM = 'from'
     TO = 'to'
@@ -67,7 +61,7 @@ class RequestMethod(Enum, AdvancedEnum, settings=NoAlias):
         return self.value
 
 
-HandlerType = Callable[[Tuple[ResponseContentType, Any], User, Optional[ResponseContextManager]], None]
+HandlerType = Callable[[Tuple[TransformerContentType, Any], User, Optional[ResponseContextManager]], None]
 
 HandlerContextType = Union[ResponseContextManager, Tuple[Optional[Dict[str, Any]], str]]
 
@@ -93,17 +87,6 @@ def int_rounded_float_typed(value: str) -> int:
     return int(round(float(value)))
 
 
-def str_response_content_type(value: str) -> ResponseContentType:
-    if value.strip() in ['application/json', 'json']:
-        return ResponseContentType.JSON
-    elif value.strip() in ['application/xml', 'xml']:
-        return ResponseContentType.XML
-    elif value.strip() in ['text/plain', 'plain']:
-        return ResponseContentType.PLAIN
-    else:
-        raise ValueError(f'"{value}" is an unknown response content type')
-
-
 class AbstractAtomicClass:
     pass
 
@@ -133,53 +116,6 @@ class AtomicVariable(Generic[T], AbstractAtomicClass):
 
         return cls.__instance
 
-    @classmethod
-    def split_value(cls, value: str, separator: str = '|') -> Tuple[str, str]:
-        return cast(Tuple[str, str], tuple([v.strip() for v in value.split(separator, 1)]))
-
-    @classmethod
-    def parse_arguments(cls, arguments: str) -> Dict[str, Any]:
-        if '=' not in arguments or (arguments.count('=') > 1 and (arguments.count('"') < 2 and arguments.count("'") < 2) and ', ' not in arguments):
-            raise ValueError(f'{cls.__name__}: incorrect format in arguments: "{arguments}"')
-
-        parsed: Dict[str, Any] = {}
-
-        for argument in arguments.split(','):
-            argument = argument.strip()
-
-            if len(argument) < 1:
-                raise ValueError(f'{cls.__name__}: incorrect format for arguments: "{arguments}"')
-
-            if '=' not in argument:
-                raise ValueError(f'{cls.__name__}: incorrect format for argument: "{argument}"')
-
-            [key, value] = argument.split('=', 1)
-
-            key = key.strip()
-            if '"' in key or "'" in key or ' ' in key:
-                raise ValueError(f'{cls.__name__}: no quotes or spaces allowed in argument names')
-
-            value = value.strip()
-
-            start_quote: Optional[str] = None
-
-            if value[0] in ['"', "'"]:
-                if value[-1] != value[0]:
-                    raise ValueError(f'{cls.__name__}: value is incorrectly quoted: "{value}"')
-                start_quote = value[0]
-                value = value[1:]
-
-            if value[-1] in ['"', "'"]:
-                if start_quote is None:
-                    raise ValueError(f'{cls.__name__}: value is incorrectly quoted: "{value}"')
-                value = value[:-1]
-
-            if start_quote is None and ' ' in value:
-                raise ValueError(f'{cls.__name__}: value needs to be quoted: "{value}"')
-
-            parsed[key] = value
-
-        return parsed
 
     @classmethod
     def get(cls) -> 'AtomicVariable[T]':
